function results = BlackScholes01(s0,K,t,T,r,sigma) ;
%  BlackScholes01(s0,K,T,r,sigma) 
%  Returns [c,p,d1,d2,Delta] (c=call price; p=put price)

d1a= log(s0./K) + (r+(sigma.^2)./2) .*(T-t)  ;
d1b =  sigma.*sqrt(T-t)  ;
d1=d1a ./ d1b ;
d2 = d1 - sigma.*sqrt(T-t) ;
c = cdfn(d1).*s0 - cdfn(d2).*K.*exp(-r.*(T-t)) ;

p = c + exp(-r.*(T-t)).*K - s0 ;
Delta_call = cdfn(d1) ;
Delta_put = Delta_call - 1 ;

gamma = normpdf(d1)/ (s0*sigma*sqrt(T-t)) ;
theta_call = -s0*sigma*normpdf(d1) / (2*sqrt(T-t)) - K*r*exp(-r*(T-t))*cdfn(d2) ;
theta_put = theta_call + r*exp(-r*(T-t))*K ;

results = struct('c',c,'p',p,'d1',d1,'d2',d2,'Delta_call',Delta_call,...
    'Delta_put',Delta_put,'gamma',gamma,'theta_call',theta_call,...
    'theta_put',theta_put) ;
