function [betahat_usual,betahat_adjusted] = ...
                          linear_eiv(yy,ww,Sigmauu);
%
% This does linear regression with measurement error.
%
% Calling Form 
%     [betahat_usual,betahat_adjusted,covmat_usual,covmat_adjusted] = ...
%                          linear_eiv(yy,ww,Sigmauu,nboot);
%
% INPUT
%      yy = response
%      ww = matrix of predictors, some measured with error
% Sigmauu = measurement error covariance matrix
%
% OUTPUT
%    betahat_usual = correction for attenuation
% betahat_adjusted = Fuller's small sample adjustment
%     covmat_usual = bootstrap covariance matrix for    usual method
%  covmat_adjusted = bootstrap covariance matrix for adjusted method
n               = size(yy,1);
m               = size(ww,2);
mzz             = cov([yy ww]);
mxy             = mzz(2(m+1),1);
jj              = 0;
while jj        < 11;
    cc          = cov(ww) - (Sigmauu .* (1 - (jj ./ 10)));
    jj          = jj + 1;
    cc          = cc + ((2 ./ (n-1)) .* Sigmauu);
    if min(eig(cc)) > 0;
        jj      = 100;
    end;
end;
betahat_usual   = inv(cc) * mxy;
Saa             = [zeros(1,m+1)
                   zeros(m,1) Sigmauu];
lambda          = 1 ./ max(eig(inv(mzz) * Saa));
cvcv            = Sigmauu;
if lambda       < 1 + (1 ./ (n-1));
    cvcv        = (lambda - (1 ./ (n-1))) .* Sigmauu;
end;
Hxx             = cov(ww) - cvcv;
betahat_adjusted = inv(Hxx + ( (2 ./ (n-1)) .* Sigmauu)) * mxy;
